#ifndef IMUTILS_INCLUDE_IMUTILS_METADBCONNECTOR_H_
#define IMUTILS_INCLUDE_IMUTILS_METADBCONNECTOR_H_

#include <vector>
#include <string>
#include <memory>

namespace erad::utils {

/**
 * Enumerated status values of resources in the meta repository
 */
enum class StatusEnum {
	/** The resource exists */
	OK,
	/** The resource has just been created */
	OK_NEW,
	/** The resource has just been copied from legacy directory */
	OK_COPIED,
	/** The resource exists in tempMeta */
	OK_TEMP,
	/** The resource does not exists */
	MISSING,
	/** An error occurred while accessing the meta repository */
	REPOSITORY_ERROR,
	/** Generic error, including when pre-existing .info direcory could not be copied to the meta repository */
	ERROR
};

/**
 * Wrapper class for status values providing conversion methods
 */
class Status {
	private:
		// The status value
		StatusEnum status;

		// Default constructor is not available
		Status() = delete;

	public:

		/**
		 * Constructs a new Status object
		 * @param status the status value
		 */
		Status( const StatusEnum& status );

		/**
		 * Returns the enumerated status value
		 * @return the status value
		 */
		inline operator StatusEnum() const {
			return status;
		}

		/**
		 * Assigns a new status value
		 *
		 * @param status the new status value
		 *
		 * @return reference to this object
		 */
		inline Status& operator=( StatusEnum status ) {
			this->status = status;
			return *this;
		}

		/**
		 * Checks if the provided status is the same as this status by comparing the enumerated values in the objects
		 * @param s the status to check
		 * @return true if and only if the enumerated status values are equal in the parameter and this objects.
		 */
		inline constexpr bool operator ==( const Status& s ) const {
			return status == s.status;
		}
		/**
		 * Checks if the provided status is different to the status value of this object by comparing the enumerated values in the objects.
		 * @param s the status to check
		 * @return true if and only if the enumerated status values are different in the parameter and this objects.
		 */
		inline constexpr bool operator !=( const Status& s ) const {
			return status != s.status;
		}
		/**
		 * Checks if the provided enumerated status value is the same as the status value of this object.
		 * @param s the status to check
		 * @return true if and only if the provided status value equals to the status value in this object.
		 */
		inline constexpr bool operator ==( StatusEnum s ) const {
			return status == s;
		}
		/**
		 * Checks if the provided enumerated status value is different to the status value of this object.
		 * @param s the status to check
		 * @return true if and only if the provided status value equals to the status value in this object.
		 */
		inline constexpr bool operator !=( StatusEnum s ) const {
			return status != s;
		}

		/**
		 * Converts the status to a string.
		 *
		 * @return a string representation of the enumerated status value
		 */
		operator std::string() const;

		/**
		 * Checks if the status indicates the resource exists
		 * @return true if and only if the status indicates the resource exists
		 */
		operator bool() const;
};
}

namespace erad::db {

using namespace erad::utils;

/**
 * Enum to differentiate between types of metadata repositories.
 * Used to determine whether the operation targets the meta repository or temp meta repository.
 */
enum class RepositoryType {
	/** Repository for storing metadata */
	META_REPOSITORY,
	/** Repository for storing deleted metadata */
	TEMP_META_REPOSITORY
};

/**
 * Interface for database connectors handling metadata operations.
 *
 * Provides an abstract interface for connecting to a database,
 * and performing basic CRUD operations on metadata documents.
 */
class MetaDBConnector {
	protected:

		/**
		 * Default constructor is only accessible from derived classes.
		 */
		MetaDBConnector() {
		}

		/* Copy and move constructors are disabled */
		MetaDBConnector( const MetaDBConnector& orig ) = delete;
		MetaDBConnector( MetaDBConnector&& orig ) = delete;

		/* Copy and move assignment operators are also disabled */
		MetaDBConnector& operator=( const MetaDBConnector& ) = delete;
		MetaDBConnector& operator=( MetaDBConnector&& ) = delete;

	public:

		/**
		 * Virtual destructor
		 */
		virtual ~MetaDBConnector() {
		}

		/**
		 * To initialize the client database and collection member variables to establish db connection.
		 *
		 * @throw mongo exception if connection failed
		 */
		virtual void connect() = 0;

		/**
		 * Method to check whether a study has meta data
		 *
		 * @param studyId parameter to search for the record
		 *
		 * @throw exception if an error occurs
		 *
		 * @return Status with below interpretation
		 *     OK				Study meta data available
		 *     REPOSITORY_ERROR Study meta data could not be located in the database
		 *     ERROR			Generic error
		 */
		virtual Status metadataExists( const std::string& studyId ) = 0;

		/**
		 * Method to retrieve the study meta data
		 *
		 * @param studyId parameter to search for the record
		 * @param docPath the specific document to retrieve the content
		 * @param content arguement to return the docPath and value in json format
		 *
		 * @throw exception if an error occurs
		 *
		 * @return an object of Status depending on the result of the operation
		 *
		 * note : if the docPath provided is present the corresonding docPath and content is returned as a pair if it is present,
		 * else if the key is empty the whole record is returned as string
		 */
		virtual Status getStudyMetaData( const std::string& studyId, const std::string& docPath,
				std::string& content ) = 0;

		/**
		 * Method to create or update record in the database
		 *
		 * @param studyId parameter to search for the record
		 * @param docPath the specific document to update the content
		 * @param content content of the document to update
		 * @param dontOverwrite not implemented
		 *
		 * @throw exception if an error occurs
		 *
		 * @return an object of Status depending on the result of the operation
		 * 
		 * note:if the corresponding studyId could not be found a new record is created with the given studyId.
		 * If an existing study is found and unless the docPath is empty, the docPath will be updated or added
		 * depending on whether it exists or not.
		 */
		virtual Status putRecordMeta( const std::string& studyId, const std::string& docPath, const std::string& content,
				const bool dontOverwrite ) = 0;

		/**
		 * Metod to find the document path which matches the filtering expression.
		 *
		 * @param studyId parameter to search for the record
		 * @param expression used to filter or match document paths associated with the specified studyId
		 * @param docPaths array of document paths which meet the filter
		 * 
		 * @throw exception if an error occurs
		 * 
		 * @return an object of Status depending on the result of the operation
		 */
		virtual Status getDocPathsWithExpression( const std::string& studyId, const std::string& expression,
				std::vector<std::string>& docPaths ) = 0;

		/**
		 * Deletes the specified document path of the given study.
		 *
		 * @param studyId parameter to search for the record
		 * @param docPath the document path which have to be removed
		 * 
		 * @throw exception if an error occurs
		 * 
		 * @return an object of Status depending on the result of the operation
		 */
		virtual Status deleteRecordDoc( const std::string& studyId, const std::string& docPath ) = 0;

		/**
		 * Delete the entire metadata of the study.
		 *
		 * @param studyId the studyId of record to be deleted
		 * 
		 * @throw exception if an error occurs
		 * 
		 * @return an object of Status depending on the result of the operation
		 */
		virtual Status deleteRecord( const std::string& studyId ) = 0;

		/**
		 * Closes the connection.
		 */
		virtual void close() = 0;

		/**
		 * This function dynamically selects and returns a `std::unique_ptr` to a
		 * database connector instance based on the configured database technology.
		 * 
		 * The actual implementation determines which concrete class to instantiate.
		 * 
		 * @throw exception if an error occurs
		 * 
		 * @return unique_ptr<MetaDBConnector> A unique pointer to the appropriate database connector instance
		 */
		static std::unique_ptr<MetaDBConnector> createConnector( erad::db::RepositoryType repositoryType );
};

}

#endif
