/*
 * This is the main interface to MongoDB.
 * It opens a connection to the DB and defines the CRUD operations.
 */
#ifndef IMUTILS_INCLUDE_IMUTILS_MONGODBCONNECTOR_H_
#define IMUTILS_INCLUDE_IMUTILS_MONGODBCONNECTOR_H_

#include "MetaDBConnector.h"
#include <exception>
#include <memory>
#include <mutex>
#include <vector>
#include <regex>
#include "bsoncxx/builder/stream/document.hpp"
#include "bsoncxx/builder/stream/helpers.hpp"
#include "mongocxx/pool.hpp"
#include "mongocxx/uri.hpp"
#include "mongocxx/exception/exception.hpp"
#include "bsoncxx/json.hpp"

namespace erad::db {

/**
 * Concrete implementation of MetaDBConnector for MongoDB.
 * Uses mongocxx driver to connect to MongoDB and perform metadata operations.
 */
class MongoDBConnector : public MetaDBConnector
{
	private:
		// Shared MongoDB connection pool used to manage and reuse client connections efficiently
		static std::unique_ptr<mongocxx::pool> m_clientPool;
		// Indicates whether the MongoDB connection pool has been initialized
		static bool m_poolInitialized;
		// Mutex to synchronize access during MongoDB pool initialization
		static std::mutex m_poolMutex;

		// MongoDB connection uri
		std::string m_connectionUri;

		// Repository configuration
		std::string m_databaseName;
		std::string m_collectionName;

	private:

		/**
		 * Function to create a document filter with the given studyId
		 *
		 * @param studyId parameter to create the filter
		 *
		 * @return document which has applied the filter
		 */
		bsoncxx::document::value createFilter( const std::string& studyId );

		/**
		 * Function to set the mongDB server URI, database and collection
		 */
		void initMongoServerCredentials();

		/**
		 * Function to create uri for connecting to the database.
		 */
		void initializePool();

		/**
		 * Function to create the client pool with the connection uri.
		 *
		 * @param connectionUri connection uri for the mongodb server
		 *
		 * @return true if the pool was create succesfully else returns false
		 */
		static bool createPool( const std::string& connectionUri );

	public:

		/**
		 * Default constructor is deleted
		 */
		MongoDBConnector() = delete;

		/**
		 * Constructor which create object based on the db required
		 */
		MongoDBConnector( erad::db::RepositoryType repositoryType );

		/**
		 * Destructor closes the connection
		 */
		~MongoDBConnector();

		/**
		 * @copydoc MetaDBConnector::connect()
		 */
		void connect() override;

		/**
		 * @copydoc MetaDBConnector::getStudyMetaData()
		 */
		Status metadataExists( const std::string& studyId ) override;

		/**
		 * @copydoc MetaDBConnector::getStudyMetaData()
		 */
		Status getStudyMetaData( const std::string& studyId, const std::string& docPath,
				std::string& content ) override;

		/**
		 * @copydoc MetaDBConnector::putRecordMeta()
		 */
		Status putRecordMeta( const std::string& studyId, const std::string& docPath, const std::string& content,
				const bool dontOverwrite ) override;

		/**
		 * @copydoc MetaDBConnector::getDocpathsWithExpression()
		 */
		Status getDocPathsWithExpression( const std::string& studyId, const std::string& expression,
				std::vector<std::string>& docPaths ) override;

		/**
		 * @copydoc MetaDBConnector::deleteRecordDoc()
		 */
		Status deleteRecordDoc( const std::string& studyId, const std::string& docPath ) override;

		/**
		 * @copydoc MetaDBConnector::deleteRecord()
		 */
		Status deleteRecord( const std::string& studyId ) override;

		/**
		 * @copydoc MetaDBConnector::close()
		 */
		void close() override;
};

} //namespace

#endif
