#ifndef CLOUDDICOMHELPER_H
#define CLOUDDICOMHELPER_H

#include <string>
#include <vector>
#include <utils/CloudDicomStore.h>
#include <nlohmann/json.hpp>

/**
 * Helper class to store Local DICOM File path and perform cleanup
 */
class LocalDicomFile {
		std::string m_absolutePath;

	public:
		LocalDicomFile( const std::string& absolutePath );
		~LocalDicomFile();

		// Delete copy constructor and assignment operator to prevent copying
		LocalDicomFile() = delete;
		LocalDicomFile( const LocalDicomFile& ) = delete;
		LocalDicomFile( const LocalDicomFile&& ) = delete;
		LocalDicomFile& operator=( const LocalDicomFile& ) = delete;
		LocalDicomFile& operator=( const LocalDicomFile&& ) = delete;

		inline const std::string& getLocalDicomFilePath() const {
			return m_absolutePath;
		}

};

/**
 * Helper class to call functions of CloudDicomStore
 */
class CloudDicomHelper {
		CloudDicomHelper();

		// Delete copy constructor and assignment operator to prevent copying
		CloudDicomHelper( const CloudDicomHelper& ) = delete;
		CloudDicomHelper( const CloudDicomHelper&& ) = delete;
		CloudDicomHelper& operator=( const CloudDicomHelper& ) = delete;
		CloudDicomHelper& operator=( const CloudDicomHelper&& ) = delete;
		
		/**
		 * Validate json Array.
		 * throw exception if not valid json
		 * 
		 * @param jsonRoot
		 *          The JSON element to be processed.
		 * @param study
		 *          Study UID.
		 *
		 * @throws Exception
		 * 
		 */
		void validateJsonArray( const nlohmann::json& jsonRoot, const std::string& study );

		/**
		 * Get the list of instances.
		 * Process dicom item and returns the list of instances.
		 * 
		 * @param dicomItem
		 *          Dicom item to process.
		 * @param sopInstanceList
		 *          sopInstanceList to fill.
		 * @param onlyPbr
		 *          Pass true, if we want only PbR List.
		 * 
		 */
		void processDicomItem( const nlohmann::json& dicomItem, std::vector<std::string>& sopInstanceList, bool onlyPbr );

		/**
		 * Get the instance list from json.
		 * 
		 * @param jsonString
		 *          json string to process.
		 * @param study
		 *          Study UID.
		 * @param onlyPbr
		 *          Pass true, if we want only PbR List.
		 * 
		 */
		std::vector<std::string> getInstanceListFromJson( const std::string& jsonString, const std::string& study,
				const bool onlyPbr = false );

		/**
		 * Parse json and get tag value.
		 * Parse the json of the instance and get the corresponding tag value from the metadata.
		 *
		 * @param json
		 *          The JSON string to be processed.
		 * @param study
		 *          Study UID.
		 * @param instance
		 *          Instance UID.
		 * @param tag
		 *          Dicom tag to get the value from json.
		 * 
		 * @return The tag value.
		 * 
		 * @throws Exception
		 */
		std::string getObjectTagValueFromJson( const std::string& jsonString, const std::string& study, const std::string& instance,
				const std::string& tag );

	public:

		static CloudDicomHelper& GetInstance() {
			static CloudDicomHelper instance;
			return instance;
		}

		/**
		 * Retrieve file from GCP
		 * 
		 * 
		 * @param dicomStore
		 *          Object of dicom store.
		 * @param basePath
		 *          base path for downloading DICOM file.
		 * @param relativeFile
		 * 			"study/series/instance"
		 * @param isPbrFile 
		 * 			true if retrieve pbr
		 * @param objectFile
		 * 			File name if isPbrFile is true
		 * 
		 * @return absolute path of the file 
		 * 
		 */
		std::unique_ptr<LocalDicomFile> retrieveFileFromSourceDevice( std::unique_ptr<CloudDicomStore> dicomStore,
				const std::string& basePath,
				const std::string& relativeFile, bool isPbrFile = false,
				const std::string& objectFile = "" );

		/**
		 * Get the PbR UID list of the study from GCP.
		 * Read the meta data of the study and returns the PbR list.
		 * 
		 * @param dicomStore
		 *          Object of dicom store.
		 * @param study
		 *          Study UID.
		 * 
		 * @return Return the PbR List.
		 * 
		 * @throws Exception
		 * 
		 */
		std::vector<std::string> getStudyPbRUIDList( CloudDicomStore& dicomStore, const std::string& study );

		/**
		 * Get the list of instances in a study from GCP.
		 * Read the meta data of the study and returns the instance list.
		 *
		 * @param dicomStore
		 *          Object of dicom store.
		 * @param study
		 *          Study UID.
		 * 
		 * @return Return the instance List.
		 * 
		 * @throws Exception
		 * 
		 */
		std::vector<std::string> getInstancesInStudy( CloudDicomStore& dicomStore, const std::string& study );

		/**
		 * Get the series UID of the instance from GCP.
		 * 
		 * @param dicomStore
		 *          Object of dicom store.
		 * @param study
		 *          Study UID.
		 * @param instance
		 *          Instance UID.
		 * 
		 * @return Return the series UID.
		 * 
		 * @throws Exception
		 * 
		 */
		std::string getObjectSeriesUID( CloudDicomStore& dicomStore, const std::string& study, const std::string& instance );
};

#endif // CLOUDDICOMHELPER_H
