#ifndef UTILS_INCLUDE_UTILS_METAMGR_H_
#define UTILS_INCLUDE_UTILS_METAMGR_H_

#include <string>
#include <set>
#include <filesystem>
#include <utils/Logger.h>
#include <imutils/DBConnector.h>
#include <imutils/MetaDBConnector.h>

namespace erad::utils {

/**
 * Manages resources in the meta repository
 * Copies legacy resources (.info directories) from the dicom repositories if needed
 */
class MetaManager {

		/// The location of the meta repository
		std::string metaRepository;

		/// The location of the tempMeta repository
		std::string tempMetaRepository;

		/// Path to the medsrv tmp directory
		std::filesystem::path tempDir;

		/// Max number of sleeps when waiting for a lock (temp director), when copying legacy resources
		size_t waitCount = 100;
		/// Max time to sleep (in milleseconds) when waiting for a lock (temp director), when copying legacy resources
		size_t sleepMillis = 10; // max 1 seconds

		/// logger for debug messages
		std::shared_ptr<Logger> debugLogger;
		/// logger for error messages
		std::shared_ptr<Logger> errorLogger;

		std::unique_ptr<std::set<std::string>> tempMetaFiles;

		/// Connection to meta database
		std::unique_ptr<erad::db::MetaDBConnector> m_metaDbConnection;

		/// Connection to temp meta database
		std::unique_ptr<erad::db::MetaDBConnector> m_tempMetaDbConnection;

	public:

		/**
		 * Constructs a new MetaManager object
		 */
		MetaManager();

		/**
		 * Destroys the MetaManager object
		 */
		~MetaManager();

		// Copy/Move constructors and assignment operators are not available

		MetaManager( const MetaManager& ) = delete;
		MetaManager( MetaManager&& ) = delete;
		MetaManager& operator=( const MetaManager& ) = delete;
		MetaManager& operator=( MetaManager&& ) = delete;

		/**
		 * Locates a resource (study) in the meta repository.
		 * Also supports returning a resource in the tempMeta repository if the resource in the meta repository does not exist.
		 * When tempMeta is requested, but not yet exists, then also checks ~/tmp/ for legacy files.
		 * Creating a resource in meta moves files from tempMeta and ~/tmp/ to the meta repository, and deletes the resource from tempMeta.
		 *
		 * @param study the resource (study) to locate
		 * @param createIfMissing if true then creates the resource if missing
		 * @param useTemp if the meta repository does not exist, then check and use the location of the tempMeta (only when createIfMissing is false)
		 * @param result [output] parameter to return the full path to the resource in the repository
		 * @param dbConn database connection
		 * @return the resource status
		 */
		Status getMeta2( const std::string& study, bool createIfMissing, bool useTemp, std::string& result,
				erad::db::DBConnector* dbConn ) const;

		[[deprecated]]
		Status getMetaDepr( const std::string& study, bool createIfMissing, bool useTemp, std::string& result,
				erad::db::DBConnector* dbConn ) const;

		/**
		 * Loads content from a meta document
		 *
		 * @param study the resource root (study) to locate
		 * @param docPath the path of the document to retrieve
		 * @param useTemp if the meta repository does not exist, then check and use the location of the tempMeta
		 * @param content [output] parameter to return the content of the document into
		 * @param dbConn database connection
		 * @return the resource status
		 */
		Status getMetaDoc( const std::string& study, const std::string& docPath, bool useTemp, std::string& content,
				erad::db::DBConnector* dbConn ) const;

		/**
		 * Writes content to a meta document
		 *
		 * @param study the resource root (study) to locate
		 * @param docPath the path of the document to retrieve
		 * @param useTemp if the meta repository does not exist, then check and use the location of the tempMeta
		 * @param content content to write into the document
		 * @param dontOverwrite only write content if doc did not exists (check Status returned)
		 * @param dbConn database connection
		 * @return the resource status
		 */
		Status putMetaDoc( const std::string& study, const std::string& docPath, bool useTemp, const std::string& content,
				bool dontOverwrite, erad::db::DBConnector* dbConn ) const;

		/**
		 * Deletes a meta document
		 *
		 * @param study the resource root (study) to locate
		 * @param docPath the path of the document to retrieve
		 * @param useTemp if the meta repository does not exist, then check and use the location of the tempMeta
		 * @param dbConn database connection
		 * @return the resource status
		 */
		Status delMetaDoc( const std::string& study, const std::string& docPath, bool useTemp, erad::db::DBConnector* dbConn ) const;

		/**
		 * Get all the docpaths with the given expression or similar to it.
		 *
		 * @param study the resource root (study) to locate
		 * @param docPathExpression the expression to find documents.
		 * @param useTemp if the meta repository does not exist, then check and use the location of the tempMeta
		 * @param dbConn database connection
		 * @param docPaths vector to store the docpaths retrieved.
		 * @return the resource status
		 */
		Status getMetaDocPaths( const std::string& study, const std::string& docPathExpression, bool useTemp,
				std::vector<std::string>& docPaths, erad::db::DBConnector* dbConn );

		/**
		 * Deletes the resource from the meta repository.
		 * If there are files to be stored 'permanently' after the meta resource is gone,
		 * then these will be moved to the TempMeta repository.
		 *
		 * @param study the resource (study) to delete
		 * @param dbConn database connection
		 * @return true if the resource could be successfully deleted
		 */
		bool deleteMeta( const std::string& study, erad::db::DBConnector* dbConn );

		/**
		 * Deletes the resource from the tempMeta repository.
		 *
		 * @param study the resource (study) to delete
		 * @param dbConn database connection
		 * @return true if the resource could be successfully deleted
		 */
		bool deleteTemp( const std::string& study, erad::db::DBConnector* dbConn ) const;

		/**
		 * Sets a new logger for the debug messages
		 * @param l the new logger for debug messages
		 */
		inline void setDebugLogger( Logger* l ) {
			this->debugLogger.reset( l );
		}

		/**
		 * Sets a new logger for the error messages
		 * @param l the new logger for error messages
		 */
		inline void setErrorLogger( Logger* l ) {
			this->errorLogger.reset( l );
		}

	private:

		Status copyLegacy( const std::string& study, const std::filesystem::path& infoDir, std::string& result,
				erad::db::DBConnector* dbConn ) const;

		void copyLegacy( const std::filesystem::path& src, const std::filesystem::path& dst ) const;

		Status restoreFromTempMeta( const std::string& study, const std::string& metaDir, erad::db::DBConnector* dbConn ) const;

		Status saveToTempMeta( const std::string& study );

		void initTempMetaFiles();

		bool moveFile( const std::filesystem::path& src, const std::filesystem::path& dst ) const;

		void moveMetaToTempMeta( const std::string& study ) const;

		/**
		 * Creates a default storestate.rec file if there was none in the legacy `.info` directory.
		 *
		 * This method does not use the StoreState class to avoid looping back to meta.
		 *
		 * @param storeStatePath path to the storestate.rec file to create
		 */
		void createDefaultStorestate( const std::filesystem::path& storeStatePath ) const;
};

}

#endif /* UTILS_INCLUDE_UTILS_METAMGR_H_ */

